def render(_init, _domain=None, xincludes=None, econtext=None, _out=None, _write=None, target_language=None, ):
	_attributes, repeat = _init.initialize_tal()
	_domain = None
	_write('<div>\n  <span>Hello, world!')
	_write('</span>')
	_write('\n')
	_write('</div>')
	_write('\n')




def render(_init, xincludes, target_language=None):
	_out, _write = _init.initialize_stream()
	_attributes, repeat = _init.initialize_tal()
	econtext = _init.initialize_scope()
	_domain = None

	# 'xinclude2.pt'
	_write('<div>\n  ')
	_include = 'xinclude2.pt'
	# %(xincludes)s.get(%(include)s, 'xml').render_xinclude(_domain=_domain, xincludes=xincludes, econtext=econtext, _out=_out, _write=_write, target_language=target_language)
	_tmp1 = xincludes.get(_include, 'xml').render_xinclude(_domain=_domain, xincludes=xincludes, econtext=econtext, _out=_out, _write=_write, target_language=target_language)
	_tmp = _tmp1
	if _tmp is not None:
		if not isinstance(_tmp, basestring):
			_tmp = str(_tmp)
		_write(_tmp)
	_write('\n')
	_write('</div>')
	_write('\n')

	return _out.getvalue()



import translation
import template
import generation
import doctypes
import etree
import config
import utils
import types

from StringIO import StringIO
from cPickle import dumps, loads

def pyexp(*exps):
    return types.parts([types.value(exp) for exp in exps])

def setup_stream(encoding=None):
    class symbols(translation.Node.symbols):
        out = '_out'
        write = '_write'

    out = StringIO()
    write = out.write
    stream = generation.CodeIO(symbols, encoding=encoding)
    return out, write, stream

def compile_xhtml(body, **kwargs):
    compiler = TestCompiler(
        body, mock_parser, implicit_doctype=doctypes.xhtml)
    return compiler(parameters=sorted(kwargs.keys()))

def render_xhtml(body, **kwargs):
    template = compile_xhtml(body, **kwargs)
    return template.render(**kwargs)    
    
def render_text(body, **kwargs):
    compiler = TestCompiler.from_text(
        body, mock_parser, implicit_doctype=doctypes.xhtml)
    template = compiler(parameters=sorted(kwargs.keys()))
    template.compile()
    return template.render(**kwargs)    

def compile_template(parser, body, encoding=None, **kwargs):
    compiler = TestCompiler(
        body, parser, encoding=encoding, implicit_doctype=doctypes.xhtml)
    template = compiler(parameters=sorted(kwargs.keys()))
    template.compile()
    return template.render(**kwargs)    

class TestCompiler(translation.Compiler):
    def __call__(self, *args, **kwargs):
        template = translation.Compiler.__call__(self, *args, **kwargs)
        template.compile()
        template = loads(dumps(template))
        return template

class MockElement(translation.Element):
    class node(translation.Node):
        def __getattr__(self, name):
            return None

        @property
        def static_attributes(self):
            return utils.get_attributes_from_namespace(
                self.element, config.XHTML_NS)

        @property
        def omit(self):
            if self.element.meta_omit is not None:
                return self.element.meta_omit or True
            if self.content:
                return True
            if self.include:
                return True

        @property
        def content(self):
            return self.element.meta_replace
        
        @property
        def cdata(self):
            return self.element.meta_cdata

        @property
        def include(self):
            href = self.element.xi_href
            if href is not None:
                return types.value(repr(href))

        @property
        def format(self):
            return self.element.xi_parse

    node = property(node)

    xi_href = None
    xi_parse = None

class MockMetaElement(MockElement, translation.MetaElement):
    pass
    
class MockXiElement(MockElement):
    xi_href = utils.attribute('href')
    xi_parse = utils.attribute("parse", default="xml")

class MockParser(etree.Parser):
    element_mapping = {
        config.XHTML_NS: {None: MockElement},
        config.META_NS: {None: MockMetaElement},
        config.XI_NS: {None: MockXiElement}}

mock_parser = MockParser()

class MockTemplate(object):
    def __init__(self, body, parser=mock_parser, doctype=None):
        self.body = body
        self.parser = parser
        self.doctype = doctype
        
    @property
    def macros(self):
        def render(name, parameters={}):
            compiler = TestCompiler(self.body, self.parser)
            template = compiler(macro=name, parameters=parameters)
            return template.render(**parameters)
        return template.Macros(render)

    def render(self, **kwargs):
        compiler = TestCompiler(
            self.body, self.parser,
            implicit_doctype=doctypes.xhtml, explicit_doctype=self.doctype)
        template = compiler(parameters=sorted(kwargs.keys()))
        return template.render(**kwargs)    

    __call__ = render



from compiler import ast, parse
from compiler.pycodegen import ModuleCodeGenerator

from transformer import ASTTransformer

import __builtin__

CONSTANTS = frozenset(['False', 'True', 'None', 'NotImplemented', 'Ellipsis'])
UNDEFINED = object()

def flatten(list):
    """Flattens a potentially nested sequence into a flat list.
    """
    l = []
    for elt in list:
        t = type(elt)
        if t is set or t is tuple or t is list or t is frozenset:
            for elt2 in flatten(elt):
                l.append(elt2)
        else:
            l.append(elt)
    return l

class Lookup(object):
    """Abstract base class for variable lookup implementations."""

    @classmethod
    def globals(cls):
        """Construct the globals dictionary to use as the execution context for
        the expression or suite.
        """
        return {
            '_lookup_attr': cls.lookup_attr,
            '_lookup_name': cls.lookup_name,
        }

    @classmethod
    def lookup_attr(cls, obj, key):
        try:
            return getattr(obj, key)
        except AttributeError, e:
            try:
                return obj[key]
            except (KeyError, TypeError, AttributeError):
                raise e

    @classmethod
    def lookup_name(cls, data, name):
        try:
            return data[name]
        except KeyError:
            raise NameError(name)

class TemplateASTTransformer(ASTTransformer):
    """Concrete AST transformer that implements the AST transformations needed
    for code embedded in templates.

    """

    def __init__(self, globals):
        self.locals = [CONSTANTS]
        builtin = dir(__builtin__)
        self.locals.append(set(globals))
        self.locals.append(set(builtin))
        # self.names is an optimization for visitName (so we don't
        # need to flatten the locals every time it's called)
        self.names = set()
        self.names.update(CONSTANTS)
        self.names.update(builtin)
        self.names.update(globals)

    def visitConst(self, node):
        #if isinstance(node.value, unicode):
        #    return ast.Const(node.value.encode('utf-8'))
        return node

    def visitAssName(self, node):
        if len(self.locals) > 1:
            if node.flags == 'OP_ASSIGN':
                self.locals[-1].add(node.name)
                self.names.add(node.name)
            else:
                self.locals[-1].remove(node.name)
                self.names.remove(node.name)
                return None
        return node

    def visitClass(self, node):
        if len(self.locals) > 1:
            self.locals[-1].add(node.name)
            self.names.add(node.name)
        self.locals.append(set())
        try:
            return ASTTransformer.visitClass(self, node)
        finally:
            self.locals.pop()

    def visitFor(self, node):
        self.locals.append(set())
        try:
            return ASTTransformer.visitFor(self, node)
        finally:
            self.locals.pop()

    def visitFunction(self, node):
        if len(self.locals) > 1:
            self.locals[-1].add(node.name)
            self.names.add(node.name)
        self.locals.append(set(node.argnames))
        self.names.update(node.argnames)
        try:
            return ASTTransformer.visitFunction(self, node)
        finally:
            self.locals.pop()

    def visitGenExpr(self, node):
        self.locals.append(set())
        try:
            return ASTTransformer.visitGenExpr(self, node)
        finally:
            self.locals.pop()

    def visitLambda(self, node):
        argnames = flatten(node.argnames)
        self.names.update(argnames)
        self.locals.append(set(argnames))
        try:
            return ASTTransformer.visitLambda(self, node)
        finally:
            self.locals.pop()

    def visitListComp(self, node):
        self.locals.append(set())
        try:
            return ASTTransformer.visitListComp(self, node)
        finally:
            self.locals.pop()

    def visitName(self, node):
        # If the name refers to a local inside a lambda, list comprehension, or
        # generator expression, leave it alone
        if not node.name in self.names:
            # Otherwise, translate the name ref into a context lookup
            func_args = [ast.Name('econtext'), ast.Const(node.name)]
            node = ast.CallFunc(ast.Name('_lookup_name'), func_args)
        return node

    def visitGetattr(self, node):
        """Get attribute with fallback to dictionary lookup.

        Note: Variables starting with an underscore are exempt
        (reserved for internal use).
        """
        
        if hasattr(node.expr, 'name') and node.expr.name.startswith('_'):
            return ast.Getattr(node.expr, node.attrname)

        return ast.CallFunc(ast.Name('_lookup_attr'), [
            self.visit(node.expr), ast.Const(node.attrname)
            ])

class Suite(object):
    __slots__ = ['code', '_globals']

    mode = 'exec'
    
    def __init__(self, source, globals=()):
        """Create the code object from a string."""

        if isinstance(source, unicode):
            source = source.encode('utf-8')
            
        node = parse(source, self.mode)

        # build tree
        transform = TemplateASTTransformer(globals)
        tree = transform.visit(node)
        filename = tree.filename = '<script>'

        # generate code
        gen = ModuleCodeGenerator(tree)
        gen.optimized = True

        self._globals = Lookup.globals()
        self.code = gen.getCode()

    def __hash__(self):
        return hash(self.code)

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.code)



from setuptools import setup, find_packages
import sys
version = '1.0b10'

install_requires = [
    'setuptools',
    'zope.interface',
    'zope.component',
    'zope.i18n >= 3.5',
    'lxml>=2.1.1',
    ]

if sys.version_info[:3] < (2,5,0):
    install_requires.append('elementtree')

setup(name='chameleon.core',
      version=version,
      description="Attribute language template compiler",
      long_description=open("README.txt").read() + open("CHANGES.txt").read(),
      classifiers=[
        "Programming Language :: Python",
        "Topic :: Text Processing :: Markup :: HTML",
        "Topic :: Text Processing :: Markup :: XML",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Malthe Borch and the Zope Community',
      author_email='zope-dev@zope.org',
      url='',
      license='BSD',
      namespace_packages=['chameleon'],
      packages = find_packages('src'),
      package_dir = {'':'src'},
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      extras_require={'lxml':['lxml>=2.1.1']},
      )



from zope import interface
from zope import component

import sys
import pprint
import config
import logging
import interfaces
import htmlentitydefs
import re, string

types = sys.modules['types']

from UserDict import UserDict

# check if we're able to coerce unicode to str
try:
    str(u'La Pe\xf1a')
    unicode_required_flag = False
except UnicodeEncodeError:
    unicode_required_flag = True
    log = logging.getLogger('Chameleon')
    log.info("Default system encoding is set to '%s'; "
             "the template engine will perform better if "
             "an encoding that coerces gracefully to "
             "unicode is used ('utf-8' recommended)." % sys.getdefaultencoding())

def coerces_gracefully(encoding):
    if encoding != sys.getdefaultencoding() and unicode_required_flag:
        return False
    return True

entities = "".join((
    '<!ENTITY %s "&#%s;">' % (name, text) for (name, text) in \
    htmlentitydefs.name2codepoint.items()))

s_counter = 0
marker = object()

def handler(key=None):
    def decorate(f):
        def g(node):
            if key is None:
                return f(node, None)
            return f(node, node.get(key))
        g.__ns__ = key
        return g
    return decorate

def attribute(tokens, factory=None, default=None, encoding=None):
    def get(self):
        for token in isinstance(tokens, tuple) and tokens or (tokens,):
            value = self.attrib.get(token)
            if value is not None:
                if encoding or self.stream.encoding:
                    value = value.encode(encoding or self.stream.encoding)
                if factory is None:
                    return value
                f = factory(self.translator)
                return f(value)

        if default is not None:
            return default
    
    return property(get)

def escape(string, quote=None, encoding=None):
    if not isinstance(string, unicode) and encoding:
        string = string.decode(encoding)
    else:
        encoding = None

    table = htmlentitydefs.codepoint2name
    def get(char):
        key = ord(char)
        if key in table:
            return '&%s;' % table[key]
        return char
    
    string = "".join(map(get, string))

    if quote is not None:
        string = string.replace(quote, '\\'+quote)

    if encoding:
        string = string.encode(encoding)
        
    return string

re_normalize = re.compile(r'(^[0-9]|\b[^A-Za-z])')
def normalize_slot_name(name):
    return re_normalize.sub('_', name)
    
def serialize(element, encoding=None):
    return "".join(serialize_element(element, encoding))

def serialize_element(element, encoding):
    try:
        name = element.tag.split('}')[-1]
    except AttributeError:
        yield element.text; return
    
    # tag opening
    yield "<%s" % name

    # attributes
    for key, value in element.attrib.items():
        yield ' %s="%s"' % (key, escape('"', encoding=encoding))

    # elements with no text which have no children are self-closing.
    if element.text is None and len(element) == 0:
        yield ' />'; return

    yield '>'
            
    if element.text is not None:
        yield element.text

    for child in element:
        for string in serialize_element(child, encoding):
            yield string

    yield '</%s>' % name

class econtext(dict):
    """Dynamic scope dictionary which is compatible with the
    `econtext` of ZPT."""
    
    set_local = setLocal = dict.__setitem__
    set_global = setGlobal = dict.__setitem__

    @property
    def vars(self):
        return self
    
class scope(list):
    def __init__(self, *args):
        global s_counter
        self.hash = s_counter
        s_counter += 1

    def __hash__(self):
        return self.hash

class emptydict(dict):
    def __setitem__(self, key, value):
        raise TypeError("Read-only dictionary does not support assignment.")
    
class repeatitem(object):
    interface.implements(interfaces.ITALESIterator)
    
    def __init__(self, iterator, length):
        self.length = length
        self.iterator = iterator
        
    @property
    def index(self):
        try:
            length = len(self.iterator)
        except TypeError:
            length = self.iterator.__length_hint__()
        except:
            raise TypeError("Unable to determine length.")

        try:
            return self.length - length - 1
        except TypeError:
            return None
            
    @property
    def start(self):
        return self.index == 0

    @property
    def end(self):
        return self.index == self.length - 1

    def number(self):
        return self.index + 1

    def odd(self):
        return bool(self.index % 2)

    def even(self):
        return not self.odd()

class repeatdict(dict):
    def insert(self, key, iterable):
        try:
            length = len(iterable)
        except TypeError:
            length = None
            
        try:
            # We used to do iterable.__iter__() but, e.g. BTreeItems
            # objects are iterable (via __getitem__) but don't possess
            # an __iter__.  call iter(iterable) instead to determine
            # iterability.
            iterator = iter(iterable)
        except TypeError:
            raise TypeError(
                "Can only repeat over an iterable object (%s)." % iterable)
        
        self[key] = (iterator, length)
        return iterator
        
    def __getitem__(self, key):
        value, length = dict.__getitem__(self, key)

        if not isinstance(value, repeatitem):
            value = repeatitem(value, length)
            self[key] = (value, length)
            
        return value

class odict(UserDict):
    def __init__(self, dict = None):
        self._keys = []
        UserDict.__init__(self, dict)

    def __delitem__(self, key):
        UserDict.__delitem__(self, key)
        self._keys.remove(key)

    def __setitem__(self, key, item):
        """Case insensitive set item."""
        
        keys = tuple(key.lower() for key in self._keys)
        _key = key.lower()
        if _key in keys:
            for k in self._keys:
                if k.lower() == _key:
                    self._keys.remove(k)
                    key = k
                    break
                
        UserDict.__setitem__(self, key, item)
        self._keys.append(key)

    def clear(self):
        UserDict.clear(self)
        self._keys = []

    def copy(self):
        dict = UserDict.copy(self)
        dict._keys = self._keys[:]
        return dict

    def items(self):
        return zip(self._keys, self.values())

    def keys(self):
        return self._keys

    def popitem(self):
        try:
            key = self._keys[-1]
        except IndexError:
            raise KeyError('dictionary is empty')

        val = self[key]
        del self[key]

        return (key, val)

    def setdefault(self, key, failobj = None):
        UserDict.setdefault(self, key, failobj)
        if key not in self._keys: self._keys.append(key)

    def update(self, dict):
        UserDict.update(self, dict)
        for key in dict.keys():
            if key not in self._keys: self._keys.append(key)

    def values(self):
        return map(self.get, self._keys)
    
def get_attributes_from_namespace(element, namespace):
    if element.nsmap.get(element.prefix, marker) in (namespace, marker):
        return dict([
            (name, value) for (name, value) in element.attrib.items() \
            if '{' not in name])

    return dict([
        (name, value) for (name, value) in element.attrib.items() \
        if name.startswith('{%s}' % namespace)])

def get_namespace(element):
    if '}' in element.tag:
        return element.tag.split('}')[0][1:]
    return element.nsmap[None]

def xhtml_attr(name):
    return "{%s}%s" % (config.XHTML_NS, name)

def tal_attr(name):
    return "{%s}%s" % (config.TAL_NS, name)

def meta_attr(name):
    return "{%s}%s" % (config.META_NS, name)

def metal_attr(name):
    return "{%s}%s" % (config.METAL_NS, name)

def i18n_attr(name):
    return "{%s}%s" % (config.I18N_NS, name)

def py_attr(name):
    return "{%s}%s" % (config.PY_NS, name)

def raise_template_exception(source, description, kwargs, exc_info):
    """Re-raise exception raised while calling ``template``, given by
    the ``exc_info`` tuple (see ``sys.exc_info``)."""

    # omit keyword arguments that begin with an underscore; these are
    # used internally be the template engine and should not be exposed
    kwargs = kwargs.copy()
    map(kwargs.__delitem__, [k for k in kwargs if k.startswith('_')])

    # format keyword arguments; consecutive arguments are indented for
    # readability
    formatted_arguments = pprint.pformat(kwargs).split('\n')
    for index, string in enumerate(formatted_arguments[1:]):
        formatted_arguments[index+1] = " "*15 + string

    # extract line number from traceback object
    cls, exc, tb = exc_info
    try:
        lineno = tb.tb_next.tb_next.tb_lineno-1
    except AttributeError:
        lineno = 1
        
    # locate source code annotation (these are available from
    # the template source as comments)
    source = source.split('\n')
    for i in reversed(range(lineno)):
        if source[i].lstrip().startswith('#'):
            annotation = source[i].split('#', 1)[-1].lstrip()
            break
    else:
        annotation = ""

    error_msg = (
        "Caught exception rendering template."
        "\n\n"
        " - Expression: \"%s\"\n"
        " - Instance:   %s\n"
        " - Arguments:  %s\n"
        ) % (annotation, description, "\n".join(formatted_arguments))    

    __dict__ = exc.__dict__
    __name__ = cls.__name__
    
    error_string = str(exc)
        
    if issubclass(cls, Exception):
        class RuntimeError(cls):
            def __init__(self, *args, **kwargs):
                pass
            
            def __str__(self):
                return "%s\n%s: %s" % (
                    error_msg, __name__, error_string)

        if isinstance(cls, types.TypeType):
            exc = RuntimeError.__new__(RuntimeError)
            exc.__dict__.update(__dict__)
        else:
            cls = RuntimeError
            
    raise cls, exc, tb
